//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------

#property copyright ""
#property link ""

#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1 Gray
#property indicator_color2 Yellow
#property indicator_color3 Aqua
#property indicator_level1 0

//
//
//
//
//

extern string TimeFrame      = "Current time frame";
extern int    RepulsePeriod1 =  1;
extern int    RepulsePeriod2 =  5;
extern int    RepulsePeriod3 = 15;
extern double SmoothLength1  =  5;
extern double SmoothPhase1   =  0;
extern double SmoothLength2  =  0;
extern double SmoothPhase2   =  0;
extern double SmoothLength3  =  0;
extern double SmoothPhase3   =  0;
extern bool   Interpolate  = true;

//
//
//
//
//

double RepulseBuffer1[];
double RepulseBuffer2[];
double RepulseBuffer3[];

//
//
//
//
//

string indicatorFileName;
bool   calculateValue;
bool   returnBars;
int    timeFrame;

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

int init()
{
   SetIndexBuffer(0,RepulseBuffer1);
   SetIndexBuffer(1,RepulseBuffer2);
   SetIndexBuffer(2,RepulseBuffer3);

   //
   //
   //
   //
   //

      indicatorFileName = WindowExpertName();
      calculateValue    = (TimeFrame=="CalculateValue"); if (calculateValue) return(0);
      returnBars        = (TimeFrame=="returnBars");     if (returnBars)     return(0);
      timeFrame         = stringToTimeFrame(TimeFrame);

   //
   //
   //
   //
   //

   IndicatorShortName(timeFrameToString(timeFrame)+" Repulse("+RepulsePeriod1+", "+RepulsePeriod2+", "+RepulsePeriod3+")");
   return(0);
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

double work[][12];
#define PosRaw1  0
#define PosRaw2  1
#define PosRaw3  2
#define NegRaw1  3
#define NegRaw2  4
#define NegRaw3  5
#define PosEma1  6
#define PosEma2  7
#define PosEma3  8
#define NegEma1  9
#define NegEma2 10
#define NegEma3 11

//
//
//
//
//

int start()
{
   int r,i,counted_bars=IndicatorCounted();

      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars - counted_bars,Bars-1);
         if (returnBars) { RepulseBuffer1[0] = limit+1; return(0); }

   //
   //
   // 
   //
   //

   if (calculateValue || timeFrame==Period())
   {
      if (ArrayRange(work,0)!=Bars) ArrayResize(work,Bars);

      double alpha1 = 2.0/(1.0+RepulsePeriod1*5.0);
      double alpha2 = 2.0/(1.0+RepulsePeriod2*5.0);
      double alpha3 = 2.0/(1.0+RepulsePeriod3*5.0);

      //
      //
      //
      //
      //
         
      for(i=limit, r=Bars-i-1; i>=0; i--,r++)
      {
         work[r][PosRaw1] = 100.0*(3*Close[i]-2*getLow(i,RepulsePeriod1)-Open[i])/Close[i];
         work[r][NegRaw1] = 100.0*(Open[i]+2*getHigh(i,RepulsePeriod1)-3*Close[i])/Close[i];
         work[r][PosEma1] = work[r-1][PosEma1]+alpha1*(work[r][PosRaw1]-work[r-1][PosEma1]);
         work[r][NegEma1] = work[r-1][NegEma1]+alpha1*(work[r][NegRaw1]-work[r-1][NegEma1]);
         
                  RepulseBuffer1[i] = iSmooth(work[r][PosEma1]-work[r][NegEma1],SmoothLength1,SmoothPhase1,i);
      
         work[r][PosRaw2] = 100.0*(3*Close[i]-2*getLow(i,RepulsePeriod2)-Open[i+RepulsePeriod2])/Close[i];
         work[r][NegRaw2] = 100.0*(Open[i+RepulsePeriod2]+2*getHigh(i,RepulsePeriod2)-3*Close[i])/Close[i];
         work[r][PosEma2] = work[r-1][PosEma2]+alpha2*(work[r][PosRaw2]-work[r-1][PosEma2]);
         work[r][NegEma2] = work[r-1][NegEma2]+alpha2*(work[r][NegRaw2]-work[r-1][NegEma2]);
         
                  RepulseBuffer2[i] = iSmooth(work[r][PosEma2]-work[r][NegEma2],SmoothLength2,SmoothPhase2,i);

         work[r][PosRaw3] = 100.0*(3*Close[i]-2*getLow(i,RepulsePeriod3)-Open[i+RepulsePeriod3])/Close[i];
         work[r][NegRaw3] = 100.0*(Open[i+RepulsePeriod3]+2*getHigh(i,RepulsePeriod3)-3*Close[i])/Close[i];
         work[r][PosEma3] = work[r-1][PosEma3]+alpha3*(work[r][PosRaw3]-work[r-1][PosEma3]);
         work[r][NegEma3] = work[r-1][NegEma3]+alpha3*(work[r][NegRaw3]-work[r-1][NegEma3]);
         
                  RepulseBuffer3[i] = iSmooth(work[r][PosEma3]-work[r][NegEma3],SmoothLength3,SmoothPhase3,i);
      }
      return(0);
   }         

   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0)*timeFrame/Period()));
   for(i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         RepulseBuffer1[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",RepulsePeriod1,RepulsePeriod2,RepulsePeriod3,SmoothLength1,SmoothPhase1,SmoothLength2,SmoothPhase2,SmoothLength3,SmoothPhase3,0,y);
         RepulseBuffer2[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",RepulsePeriod1,RepulsePeriod2,RepulsePeriod3,SmoothLength1,SmoothPhase1,SmoothLength2,SmoothPhase2,SmoothLength3,SmoothPhase3,1,y);
         RepulseBuffer3[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",RepulsePeriod1,RepulsePeriod2,RepulsePeriod3,SmoothLength1,SmoothPhase1,SmoothLength2,SmoothPhase2,SmoothLength3,SmoothPhase3,2,y);

         //
         //
         //
         //
         //
      
            if (!Interpolate || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;

         //
         //
         //
         //
         //

         datetime time = iTime(NULL,timeFrame,y);
            for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
            for(int k = 1; k < n; k++)
            {
               RepulseBuffer1[i+k] = RepulseBuffer1[i] + (RepulseBuffer1[i+n]-RepulseBuffer1[i])*k/n;
               RepulseBuffer2[i+k] = RepulseBuffer2[i] + (RepulseBuffer2[i+n]-RepulseBuffer2[i])*k/n;
               RepulseBuffer3[i+k] = RepulseBuffer3[i] + (RepulseBuffer3[i+n]-RepulseBuffer3[i])*k/n;
            }               
   }
   return(0);         
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//    using period+1 is wrong but I left it to stay the same as the original
//    if we use the usual logic the "+1" part is an error
//
//

double getLow(int from, int period)
{
   return(Low[iLowest(NULL,0,MODE_LOW,period+1,from)]);
}
double getHigh(int from, int period)
{
   return(High[iHighest(NULL,0,MODE_HIGH,period+1,from)]);
}



//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double wrk[][10];

#define bsmax  5
#define bsmin  6
#define volty  7
#define vsum   8
#define avolty 9


//
//
//
//
//

double iSmooth(double price, double length, double phase, int i, int s=0)
{
   if (length <=1) return(price);
   if (ArrayRange(wrk,0) != Bars) ArrayResize(wrk,Bars);
   
   int r = Bars-i-1; 
      if (r==0) { for(int k=0; k<7; k++) wrk[r][k+s]=price; for(; k<10; k++) wrk[r][k+s]=0; return(price); }

   //
   //
   //
   //
   //
   
      double len1   = MathMax(MathLog(MathSqrt(0.5*(length-1)))/MathLog(2.0)+2.0,0);
      double pow1   = MathMax(len1-2.0,0.5);
      double del1   = price - wrk[r-1][bsmax+s];
      double del2   = price - wrk[r-1][bsmin+s];
      double div    = 1.0/(10.0+10.0*(MathMin(MathMax(length-10,0),100))/100);
      int    forBar = MathMin(r,10);
	
         wrk[r][volty+s] = 0;
               if(MathAbs(del1) > MathAbs(del2)) wrk[r][volty+s] = MathAbs(del1); 
               if(MathAbs(del1) < MathAbs(del2)) wrk[r][volty+s] = MathAbs(del2); 
         wrk[r][vsum+s] =	wrk[r-1][vsum+s] + (wrk[r][volty+s]-wrk[r-forBar][volty+s])*div;
         
         //
         //
         //
         //
         //
   
         wrk[r][avolty+s] = wrk[r-1][avolty+s]+(2.0/(MathMax(4.0*length,30)+1.0))*(wrk[r][vsum+s]-wrk[r-1][avolty+s]);
            if (wrk[r][avolty+s] > 0)
               double dVolty = wrk[r][volty+s]/wrk[r][avolty+s]; else dVolty = 0;   
	               if (dVolty > MathPow(len1,1.0/pow1)) dVolty = MathPow(len1,1.0/pow1);
                  if (dVolty < 1)                      dVolty = 1.0;

      //
      //
      //
      //
      //
	        
   	double pow2 = MathPow(dVolty, pow1);
      double len2 = MathSqrt(0.5*(length-1))*len1;
      double Kv   = MathPow(len2/(len2+1), MathSqrt(pow2));

         if (del1 > 0) wrk[r][bsmax+s] = price; else wrk[r][bsmax+s] = price - Kv*del1;
         if (del2 < 0) wrk[r][bsmin+s] = price; else wrk[r][bsmin+s] = price - Kv*del2;
	
   //
   //
   //
   //
   //
      
      double R     = MathMax(MathMin(phase,100),-100)/100.0 + 1.5;
      double beta  = 0.45*(length-1)/(0.45*(length-1)+2);
      double alpha = MathPow(beta,pow2);

         wrk[r][0+s] = price + alpha*(wrk[r-1][0+s]-price);
         wrk[r][1+s] = (price - wrk[r][0+s])*(1-beta) + beta*wrk[r-1][1+s];
         wrk[r][2+s] = (wrk[r][0+s] + R*wrk[r][1+s]);
         wrk[r][3+s] = (wrk[r][2+s] - wrk[r-1][4+s])*MathPow((1-alpha),2) + MathPow(alpha,2)*wrk[r-1][3+s];
         wrk[r][4+s] = (wrk[r-1][4+s] + wrk[r][3+s]); 

   //
   //
   //
   //
   //

   return(wrk[r][4+s]);
}